#
# Python S(q,w) module for anticrossing
# @author Petr Cermak and Tobias Weber
# @license GPLv2
# @date jun-2016
#

import math as m

import numpy as np
import numpy.linalg as la

from numpy import array,sqrt,cos,pi

import scipy as sp
import scipy.constants as const



# -----------------------------------------------------------------------------
# dispersion
# -----------------------------------------------------------------------------

# kB in meV/K
kB = const.k / const.e * 1e3


# dispersion relations
def myDispersion(x, A, E, G, SIGN):
    """
    Simple anticrossign of two dispersions
    x - position in bz (r.l.u.)
    A - Amplitude of phonon branch (meV)
    E - Energy of flat branch (meV)
    G - coupling constant (meV^2)
    """
    phonon = A*sqrt(1-cos(x*pi))
    a = (E**2 - phonon**2)**2 / 4 + 2*E*G*phonon
    b = (E**2 + phonon**2)/2 + SIGN * sqrt(a) 
    return sqrt(b)


# Gaussian peak
def gauss(x, x0, sig, amp):
    norm = (np.sqrt(2.*m.pi) * sig)
    return amp * np.exp(-0.5*((x-x0)/sig)**2.) / norm

# Bose factor
def bose(E, T):
    if E >= 0.:
        return 1./(m.exp(abs(E)/(kB*T)) - 1.) + 1.;
    else:
        return 1./(m.exp(abs(E)/(kB*T)) - 1.);

# Bose factor which is cut off below Ecut
def bose_cutoff(E, T, Ecut=0.02):
    Ecut = abs(Ecut)
    b = bose(E, T)

    if abs(E) < Ecut:
        bcut = bose(np.sign(E)*Ecut, T)
        b = bcut

    return b

# -----------------------------------------------------------------------------
# Takin interface
# -----------------------------------------------------------------------------

# global variables which can be accessed / changed by Takin
g_peak = np.array([2., 0., 2.])	# Bragg peak

g_CF_E = 4.         # cf level
g_E_phon = 5.       # energy of phonon
g_G = 0.5         # coupling constant

g_sig_ph = 0.2     # linewidth phonon
g_S0_ph = 1.        # intensity phonon
g_sig_CF = 0.3      # linewidth CF
g_S0_CF = 20.0         # intensity CF

g_mix_sig = 0.5   # mixing linewidth (same as CF?)


g_inc_sig = 0.02    # incoherent width
g_inc_amp = 0.1      # incoherent intensity

g_T = 300.          # temperature

g_b_cut = 0.02      # cutoff energy for Bose factor


# the init function is called after Takin has changed a global variable
def TakinInit():
    print("Init: G=" + repr(g_G) + ", T=" + repr(g_T))

# called for every Monte-Carlo point
def TakinSqw(h, k, l, E):
    try:
        Q = np.array([h,k,l])
        q = la.norm(Q - g_peak)

        E_peak_t = myDispersion(q, g_E_phon, g_CF_E, g_G, -1)
        E_peak_b = myDispersion(q, g_E_phon, g_CF_E, g_G, 1)

        S_t_ph = gauss(E, E_peak_t, g_sig_ph, g_S0_ph)
        S_b_ph = gauss(E, E_peak_b, g_sig_ph, g_S0_ph)
        
        S_t_CF = gauss(E, E_peak_t, g_sig_CF, g_S0_CF)
        S_b_CF = gauss(E, E_peak_b, g_sig_CF, g_S0_CF)
        
        #determine mixing
        cf_part = gauss(E, g_CF_E, g_mix_sig, g_mix_sig * sqrt(2*pi))
        
        
        incoh = gauss(E, 0., g_inc_sig, g_inc_amp)

        S = ((1-cf_part) * (S_t_ph + S_b_ph) + (cf_part) * (S_t_CF + S_b_CF))*bose_cutoff(E, g_T, g_b_cut) + incoh
        #S = ((cf_part) * (S_t_CF + S_b_CF))
        
        #print("S={0}".format(S))
        return S
    except ZeroDivisionError:
        return 0.

# -----------------------------------------------------------------------------